package com.qmth.cqb.paper.service.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import javax.persistence.criteria.Predicate;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import com.netflix.infix.lang.infix.antlr.EventFilterParser.null_predicate_return;
import com.qmth.cqb.paper.dao.PaperDetailInfoStructBlueprintRepo;
import com.qmth.cqb.paper.dao.PaperDetailStructBlueprintRepo;
import com.qmth.cqb.paper.dao.PaperStructBlueprintRepo;
import com.qmth.cqb.paper.dto.PaperStructBlueprintDto;
import com.qmth.cqb.paper.model.PaperDetailInfoStructBlueprint;
import com.qmth.cqb.paper.model.PaperDetailStructBlueprint;
import com.qmth.cqb.paper.model.PaperStructBlueprint;
import com.qmth.cqb.paper.service.PaperDetailStructBlueprintService;
import com.qmth.cqb.paper.service.PaperStructBlueprintService;
import com.rabbitmq.client.AMQP.Basic.Return;

/**
 * @describle 蓝图试卷结构  coursePropertyServiceImpl
 * @author weiwenhai
 * @date   2017.11.17
 */
@Service("paperStructBlueprintService")
public class PaperStructBlueprintServiceImpl implements PaperStructBlueprintService {

	@Autowired
	private PaperStructBlueprintRepo paperStructBlueprintRepo;
	
	@Autowired
	private PaperDetailStructBlueprintRepo paperDetailStructBlueprintRepo;
	
	@Autowired
	private PaperDetailInfoStructBlueprintRepo paperDetailInfoStructBlueprintRepo;
	
	@Autowired
	private PaperDetailStructBlueprintService paperDetailStructBlueprintService;
	
	@Override
	public Page<PaperStructBlueprint> findAll(PaperStructBlueprintDto paperStructBlueprintDto, Pageable pageable) {
		Specification<PaperStructBlueprint> specification = getSpecification(paperStructBlueprintDto);
		Page<PaperStructBlueprint> paperStructBlueprintsPage = paperStructBlueprintRepo.findAll(specification, pageable);
		return paperStructBlueprintsPage;
	}

	private Specification<PaperStructBlueprint> getSpecification(PaperStructBlueprintDto paperStructBlueprintDto) {
		PaperStructBlueprint paperStructBlueprint = new PaperStructBlueprint(paperStructBlueprintDto);
		Specification<PaperStructBlueprint> specification = (root, query, cb) -> {
			List<Predicate> predicates = new ArrayList<>();
			if(!StringUtils.isEmpty(paperStructBlueprint.getId())){
				predicates.add(cb.equal(root.get("orgId"),paperStructBlueprint.getOrgId()));
			}
			if(!StringUtils.isEmpty(paperStructBlueprint.getName())){
		    	predicates.add(cb.like(root.get("name"),"%"+paperStructBlueprint.getName()+"%"));
		    }
			if(!StringUtils.isEmpty(paperStructBlueprint.getCourseNo())){
		    	predicates.add(cb.equal(root.get("courseNo"),paperStructBlueprint.getCourseNo()));
		    }
			return cb.and(predicates.toArray(new Predicate[predicates.size()]));
		};
		return specification;
	}

	@Override
	public void saveSingle(PaperStructBlueprintDto paperStructBlueprintDto, String PaperDetailIds) {
		PaperStructBlueprint paperStructBlueprint = new PaperStructBlueprint(paperStructBlueprintDto);
		List<Long> ids = Stream.of(PaperDetailIds.split(",")).map(s->Long.parseLong(s.trim())).collect(Collectors.toList());
		if(ids == null || ids.size()<0){
			paperStructBlueprint.setDetailCount(0);
		}else{
			paperStructBlueprint.setDetailCount(ids.size());
			//计算试卷结构难度
			Double difficulty = getDifficulty(ids);
			paperStructBlueprint.setDifficulty(difficulty);
			paperStructBlueprint.setCreateTime(new Date());
			paperStructBlueprint = paperStructBlueprintRepo.save(paperStructBlueprint);
			for(Long id:ids){
				PaperDetailStructBlueprint paperDetailStructBlueprint = paperDetailStructBlueprintRepo.findOne(id);
				paperDetailStructBlueprint.setPaperStructBlueprintId(paperStructBlueprint.getId());
				paperDetailStructBlueprintRepo.save(paperDetailStructBlueprint);
			}
		}
	}

	@Override
	public Double getDifficulty(List<Long> ids) {
		if(ids != null && ids.size() >0){
			Integer simpleCount = 0;
			Integer mediumCount = 0;
			Integer difficultyCount = 0;
			for(Long id:ids){
				PaperDetailInfoStructBlueprint paperDetailInfoStructBlueprint = paperDetailInfoStructBlueprintRepo.findByPaperDetailStructBlueprintId(id);
				simpleCount = paperDetailInfoStructBlueprint.getPublicSimpleCount() + paperDetailInfoStructBlueprint.getNoPublicSimpleCount();
				mediumCount = paperDetailInfoStructBlueprint.getPublicMediumCount() + paperDetailInfoStructBlueprint.getNoPublicMediumCount();
				difficultyCount = paperDetailInfoStructBlueprint.getPublicDifficultyCount() + paperDetailInfoStructBlueprint.getNoPublicDifficultyCount();
			}
			Double difficulty = ((simpleCount * 0.8) + (mediumCount * 0.5) + (difficultyCount * 0.2))/(simpleCount + mediumCount + difficultyCount);
			return difficulty;
		}
		return (double)0;
	}

	@Override
	public void deleteSingle(Long id) {
		//查询大题数量
		List<PaperDetailStructBlueprint> paperDetailStructBlueprints = paperDetailStructBlueprintRepo.findByPaperStructBlueprintId(id);
		for(PaperDetailStructBlueprint paperDetailStructBlueprint:paperDetailStructBlueprints){
			//删除每个大题下面的小题
			paperDetailStructBlueprintService.deleteSingle(paperDetailStructBlueprint.getId());
		}
		paperStructBlueprintRepo.delete(id);
	}

	@Override
	public List<PaperStructBlueprint> findByCourseNo(String courseNo) {
		List<PaperStructBlueprint> paperStructBlueprints = paperStructBlueprintRepo.findByCourseNo(courseNo);
		return paperStructBlueprints;
	}

}
